SystemOrganization addCategory: #'Zippers-Support'!
SystemOrganization addCategory: #Zippers!
SystemOrganization addCategory: #'Zippers-BST'!
SystemOrganization addCategory: #'Zippers-Tests'!

Error subclass: #ZipperNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperNavigation commentStamp: 'fbs 12/28/2010 17:36' prior: 0!
Raise me whenever a Zipper tries to navigate to somewhere it cannot - past the end of a list, to a missing subtree, etc.!

TestCase subclass: #BinarySearchTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 13:33'!
testAddToEmptyTreeAdds
	| items tree |
	tree := (BinarySearchTree tree: ZTree empty) add: 1.
	items := OrderedCollection new.
	PreOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingLargerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 3.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := ((BinarySearchTree with: 1) add: 3) add: 2.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1 2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingSmallerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 1.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1 2) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/26/2011 22:50'!
testCollectExecutesBlockOncePerNode
	| items t |
	t := ((BinarySearchTree with: 2) add: 3) add: 1.
	items := t collect: [:each | each * 2].
	self assert: items = (#(1 2 3) * 2) asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/24/2011 17:32'!
testNewTreeIsBlank
	self assert: BinarySearchTree new asOrderedCollection = #() asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - instance creation' stamp: 'fbs 2/24/2011 22:52'!
testWithReturnsSingleNodeTree
	| bst value |
	value := 1.
	bst := BinarySearchTree with: value.
	self assert: (bst collect: [:each | each]) = #(1) asOrderedCollection.
	! !

TestCase subclass: #ListZipperTest
	instanceVariableNames: 'prefix suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangeChangesValue
	| list zipper |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangePreservesOriginal
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right.
	zipper := zipper change: [:x | x size].
	zipper := zipper left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.
	self assert: list = original description: 'Zipper not functional'.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:15'!
testChangeSharesStructure
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 (2 3) 5) asOrderedCollection.
	self assert: zipper focus second == original second description: 'Zipper not sharing original structure'.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testLeft
	| left list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right right.
	
	left := right left.
	self assert: left focus = list allButFirst.
	
	left := left left.
	self assert: left focus = list.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testLeftPastBeginningOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right left left ]
		raise: ZipperNavigation.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:49'!
testOn
	| list zipper |
	list := #(1 2 3 4 5).
	zipper := ListZipper on: list.
	self assert: zipper focus = list asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testRight
	| list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right.
	self assert: right focus = list allButFirst.
	
	right := right right.
	self assert: right focus = list allButFirst allButFirst.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testRightPastEndOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right right right ]
		raise: ZipperNavigation.! !

TestCase subclass: #TraversalTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

TraversalTestCase subclass: #InOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!InOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/24/2011 17:37'!
makeTraversal
	^ InOrderTraversal new.! !

!InOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/24/2011 17:38'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal collect: [:each | each] using: tree zipper.
	
	self assert: i = #(3 2 4 1 5) asOrderedCollection.! !

TraversalTestCase subclass: #PostOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PostOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/18/2011 07:24'!
makeTraversal
	^ PostOrderTraversal new.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/21/2011 18:02'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal collect: [:each | each * 2] using: tree zipper.
	
	self assert: i = (#(3 4 2 5 1) * 2) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/21/2011 18:01'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal collect: [:each | each] using: tree zipper.
	
	self assert: i = #(3 4 2 5 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:26'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(3 4 2 5 1).! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:51'!
testTreeTraversalMovesDepthFirstOnTriangleTree
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3})
		 yieldsOrder: #(2 3 1).! !

TraversalTestCase subclass: #PreOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PreOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ PreOrderTraversal new.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/25/2011 17:41'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal collect: [:each | each * 2] using: tree zipper.
	
	self assert: i = (#(1 2 3 4 5) * 2) asOrderedCollection.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:11'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(1 2 3 4 5).! !

!TraversalTestCase class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TraversalTestCase.! !

!TraversalTestCase methodsFor: 'testing' stamp: 'fbs 2/17/2011 22:08'!
assertTraversalOf: aTree yieldsOrder: aCollection
	| i |
	i := #() asOrderedCollection.
	self makeTraversal traverse: aTree doing: [:each | i add: each ].
	
	self assert: i = aCollection asOrderedCollection.! !

!TraversalTestCase methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ self subclassResponsibility.! !

!TraversalTestCase methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
setUp
	traversal := self makeTraversal.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testEmptyTraversalDoesNothing
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/25/2011 16:45'!
testEnumeratingEmptyTreeReturnsEmptyCollection
	| items |
	items := self makeTraversal collect: [:each | each ] using: ZTree empty zipper.
	self assert: items isEmpty.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testTraverseBranchingTreeCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) doing: [:each | i := i + 1].
	
	self assert: i = 3.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:06'!
testTreeTraversalCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

TestCase subclass: #TreeTestCases
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeTestCases class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TreeTestCases.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:35'!
emptyTree
	^ ZTree empty.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 07:59'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TreeTestCases subclass: #TreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:36'!
emptyTree
	^ ZTree empty.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 18:21'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:05'!
makeZipperOn: aTree
	^ TreeZipper on: aTree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 17:58'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 18:47'!
testChangeReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper change: [ :i | i + 1].
	self assert: zipper focus value = 3 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 19:31'!
testChangeToReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper changeTo: 5.
	self assert: zipper focus value = 5 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - copying' stamp: 'fbs 2/25/2011 08:06'!
testCopyReturnsExactCopy
	| copy zipper |
	zipper := (self makeZipperOn: self triangleTree) down.
	copy := zipper copy.
	
	self deny: copy == zipper.
	self assert: copy focus = zipper focus.
	self assert: copy up focus = zipper up focus.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 15:55'!
testDeleteAtTopFails
	self should: [self emptyTree zipper delete] raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteLeafNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:07'!
testDeleteLeafNodeDeletesNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:15'!
testDeleteLeafNodeFocusesOnParent
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down.
	zipper := zipper delete.
	
	self assert: zipper root = (ZTree value: 1).
	self assert: zipper focus value = 1.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteNodeFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteOldestSiblingFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down.
	self assert: zipper focus value = 2.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteYoungerSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right.
	self assert: zipper focus value = 3.
	zipper := zipper delete.
	
	self assert: zipper focus value = 2 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:16'!
testDeleteYoungestSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right right.
	self assert: zipper focus value = 4.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:18'!
testDownOnEmptyTreeSignalsException
	| zipper |
	zipper := TreeZipper on: self emptyTree.
	
	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:59'!
testDownOnLeafNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:44'!
testDownOnNonEmptyTreeFocusesOnFirstSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	
	self assert: zipper focus = tree children first.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:42'!
testDownOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper downOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 07:50'!
testDownOrElseOnEmptyTreeRunsBlock
	| ran zipper |
	zipper := self emptyTree zipper.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:18'!
testDownOrElseOnLeafNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:29'!
testDownOrElseOnNonEmptyTreeFocusesOnFirstSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	
	self assert: zipper focus = tree children first.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:46'!
testInsertDownInsertsChild
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	
	zipper := zipper insertDown: 1.
	"self should: [zipper up] raise: ZipperNavigation."
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 2/24/2011 17:29'!
testInsertDownOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := tree zipper.
	
	zipper := zipper insertDown: 0.
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	self should: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:47'!
testInsertLeftInsertsImmediatelyOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}.
	zipper := tree zipper.
	zipper := zipper down right insertLeft: 3.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 3. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:20'!
testInsertLeftInsertsOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertLeft: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 2. ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:52'!
testInsertRightInsertsYoungerSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertRight: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:38'!
testIsEmptyOnEmptyTreeReturnsTrue
	self assert: self emptyTree zipper isEmpty.! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:42'!
testIsEmptyOnNonEmptyTreeAlwaysReturnsFalse
	| zipper |
	zipper := self triangleTree zipper.
	self deny: zipper isEmpty.
	
	self deny: zipper down isEmpty.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:48'!
testLeftAtLeftmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 17:29'!
testLeftAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:22'!
testLeftMovesToLeftSibling
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	zipper := zipper left.
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtLeftmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testLeftOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper leftOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:34'!
testLeftOrElseMovesToLeftSibling
	| ran tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	ran := false.
	zipper := zipper leftOrElse: [ran := true].
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:34'!
testLeftRightIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	4 timesRepeat: [zipper := zipper right].
	self assert: zipper focus value = 6.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated left ].
			i timesRepeat: [ navigated := navigated right ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2010 22:19'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := TreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:49'!
testRightAtRightmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:42'!
testRightAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:35'!
testRightLeftIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated right ].
			i timesRepeat: [ navigated := navigated left ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:27'!
testRightMovesToRightSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down right.
	
	self assert: zipper focus = (tree children at: 2).! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRightmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testRightOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:41'!
testRightOrElseMovesToRightSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: zipper focus = (tree children at: 2).
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:48'!
testRightStoresValueCorrectly
	"#right didn't store the value correctly, and would cause the right-most node
	value to be written to all the subsequently-visited nodes."
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := tree zipper.
	"Down, any (valid) combination of right/left, then up: no-op."
	self assert: zipper focus value = zipper down right up focus value.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:31'!
testRootAtChildNodeReturnsRoot
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper down.
	
	self assert: zipper root = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:29'!
testRootAtRootReturnsRoot
	| tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	
	self assert: zipper root == tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpAtRootNodeSignalsException
	| zipper |
	zipper := self triangleTree zipper.
	
	self
		should: [zipper up]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:14'!
testUpOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := self triangleTree zipper.
	
	ran := false.
	zipper := zipper upOrElse: [ran := true.].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testUpOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:20'!
testUpOrElseReturnsToParentNode
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:21'!
testUpOrElseReturnsToParentNode2
	| ran tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down down up upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpReturnsToParentNode
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 14:46'!
testUpReturnsToParentNode2
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	zipper := zipper down down up up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 16:53'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TestCase subclass: #ZTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsEqualSingleNodesReturnsTrue
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:36'!
testEqualsEqualTreesReturnsTrue
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsUnequalSingleNodesReturnsFalse
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 2.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:37'!
testEqualsUnequalTreesInChildrenReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 1}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:39'!
testEqualsUnequalTreesInValueReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 2 children: {ZTree value: 2}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 2/25/2011 21:33'!
testPrintEmptyTreeShowsSpecialSymbol
	self assert: '<>' = ZTree empty printString.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:32'!
testPrintSingleNodePrintsNoChildren
	self assert: (ZTree value: 1) printString = '(1)'.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:33'!
testPrintTreeNodePrintsChildren
	| t |
	t := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	self assert: t printString = '(1 (2), (3))'.! !

!ZTreeTests methodsFor: 'testing - instance creation' stamp: 'fbs 2/25/2011 17:55'!
testValueWithEmptyTreeReturnsEmptyTree
	self assert: ZTree empty == (ZTree value: ZTree empty).! !

Object subclass: #BinarySearchTree
	instanceVariableNames: 'tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-BST'!

!BinarySearchTree commentStamp: 'fbs 2/5/2011 23:10' prior: 0!
I implement a binary search tree in a strictly functional manner.!

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 2/21/2011 22:12'!
tree: aTree
	^ self new tree: aTree.! !

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 2/23/2011 21:04'!
with: anObject
	^ self new tree: (ZTree value: anObject).! !

!BinarySearchTree methodsFor: 'adding' stamp: 'fbs 2/21/2011 22:12'!
add: aComparableObject
	"Add some object that understands < to our tree. aComparableObject < node goes into the left subtree; otherwise, the object goes into the right subtree."
	^ BinarySearchTree tree: (self add: aComparableObject to: tree zipper)! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/25/2011 22:56'!
add: anObject to: aZipper
	"If we're at a leaf node, and <, insertDown."
	"If we're not at a leaf node, move further into the tree."
	| currentValue leftChild move |
	aZipper focus isEmpty ifTrue: [ ^ (aZipper insertDown: anObject) root ].
	
	currentValue := aZipper focus value.
	
	"Move down to the left subnode."
	move := aZipper safeDown.
	move first = #success ifFalse:
		["Leaf node. If adding a right subtree, add the empty tree as a left subtree."
		^ (anObject < currentValue
			ifTrue: [ aZipper insertDown: anObject ]
			ifFalse: [ (aZipper insertDown: ZTree empty) insertRight: anObject ]) root ].

	leftChild := move second.
	
	"If less than, add to the left subtree."
	anObject < currentValue
		ifTrue: [ ^ self add: anObject to: leftChild ].
		
	"Otherwise, add to the right subtree."
	move := leftChild safeRight.
	^ self add: anObject to: move second.! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 2/24/2011 17:32'!
asOrderedCollection
	^ self collect: [:each | each ].! !

!BinarySearchTree methodsFor: 'enumerating' stamp: 'fbs 2/26/2011 22:53'!
collect: aBlock
	^ InOrderTraversal new collect: aBlock using: tree zipper.! !

!BinarySearchTree methodsFor: 'initialize-release' stamp: 'fbs 2/24/2011 17:33'!
initialize
	tree := ZTree empty.! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/21/2011 22:12'!
tree: aTree
	tree := aTree.! !

!BinarySearchTree methodsFor: 'accessing' stamp: 'fbs 2/25/2011 21:36'!
zipper
	^ tree zipper.! !

Object subclass: #DepthTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!DepthTraversal methodsFor: 'private' stamp: 'fbs 2/18/2011 07:28'!
depthFirstTraversal: aZipper doing: aBlock
	self subclassResponsibility.! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 2/21/2011 07:24'!
traverse: aTree doing: aBlock
	^ self depthFirstTraversal: aTree zipper doing: aBlock.! !

DepthTraversal subclass: #InOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!InOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/24/2011 22:42'!
collect: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| left move right value |
	aZipper focus isEmpty ifTrue: [^ OrderedCollection new ].
	
	value := aBlock value: aZipper focus value.
	move := aZipper safeDown.
	
	left := (move first = #success)
		ifTrue: [ self collect: aBlock using: move second ]
		ifFalse: [ #() asOrderedCollection ].
	
	right := ((move first = #success) and: [(move := move second safeRight) first = #success])
		ifTrue: [self collect: aBlock using: move second]
		ifFalse: [ #() asOrderedCollection ].

	^ left copy
		add: value;
		addAll: right;
		yourself.! !

!InOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 10:18'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.! !

DepthTraversal subclass: #PostOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PostOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/25/2011 17:33'!
collect: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected move value |
	aZipper isEmpty ifTrue: [ ^ OrderedCollection new ].
	
	value := aBlock value: aZipper focus value.
	move := aZipper safeDown.
	collected := (move first = #success)
		ifTrue: [ (self collect: aBlock using: move second) copy add: value; yourself ]
		ifFalse: [ { value } asOrderedCollection ].
	
	move := aZipper safeRight.
	move first = #success ifTrue: [collected addAll: (self collect: aBlock using: move second)].

	^ collected.! !

!PostOrderTraversal methodsFor: 'private' stamp: 'fbs 2/21/2011 18:01'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.
! !

DepthTraversal subclass: #PreOrderTraversal
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PreOrderTraversal commentStamp: 'fbs 2/13/2011 17:04' prior: 0!
I provide a pre-order traversal of an n-tree: I visit the value of a node and then its subchildren.!

!PreOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/25/2011 17:44'!
collect: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected move value |
	aZipper isEmpty ifTrue: [ ^ OrderedCollection new ].
	
	value := aBlock value: aZipper focus value.
	move := aZipper safeDown.
	collected := (move first = #success)
		ifTrue: [ (self collect: aBlock using: move second) copy addFirst: value; yourself ]
		ifFalse: [ { value } asOrderedCollection ].
	
	move := aZipper safeRight.
	move first = #success ifTrue: [collected addAll: (self collect: aBlock using: move second)].

	^ collected.! !

!PreOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 17:37'!
depthFirstTraversal: aZipper doing: aBlock
	"A naive depth-first traversal. It fails in the presence of cycles."
	^ self collect: aBlock using: aZipper.
	
	"We're backtracking."
	"
	aZipper ifNil: [^ nil ].
	
	aBlock value: aZipper focus value.
	
	aZipper isEmpty
		ifTrue: [ ^ nil ].

	^ self depthFirstTraversal:
			(aZipper downOrElse:
				[self depthFirstTraversal:
					(aZipper rightOrElse: [
						self depthFirstTraversal:
							(aZipper up rightOrElse: [ nil ])
							doing: aBlock ])
					doing: aBlock ])
			doing: aBlock."! !

Object subclass: #ListZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ListZipper commentStamp: 'fbs 12/28/2010 16:27' prior: 0!
A zipper is a structure that allows one, in a purely functional manner, to navigate a structure. It works by remembering the steps taken to focus on any particular part of a structure.

A ListZipper is a zipper for any sequenceable collection. It has two instance variables, namely

focus - the (sub)list on which we are currently focussed.
trail - the steps taken to focus on the focus instance variable.!

!ListZipper class methodsFor: 'instance creation' stamp: 'fbs 12/28/2010 16:48'!
on: aSequenceableCollection
	^ self new focus: aSequenceableCollection asOrderedCollection trail: OrderedCollection new.! !

!ListZipper methodsFor: 'mutating' stamp: 'fbs 12/30/2010 20:50'!
change: aBlock
	"Given an arity 1 block, replace the current value in the list with that of evaluating aBlock on that value."
	
	^ ListZipper new
		focus: (aBlock value: focus first) asOrderedCollection, focus allButFirst
		trail: trail.! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/28/2010 16:27'!
focus
	^ focus.! !

!ListZipper methodsFor: 'private' stamp: 'fbs 12/28/2010 16:36'!
focus: aSeqColl trail: anotherSeqColl
	focus := aSeqColl.
	trail := anotherSeqColl.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
left
	trail isEmpty ifTrue: [^ ZipperNavigation new signal: 'Left at the first element'].
	
	^ ListZipper new
		focus: (focus copy addFirst: trail first; yourself)
		trail: trail allButFirst.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
right
	focus isEmpty ifTrue: [^ ZipperNavigation new signal: 'Right at the last element'].
	
	^ ListZipper new
		focus: focus allButFirst
		trail: (trail copy addFirst: focus first; yourself)! !

Object subclass: #TreeZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipper commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a zipper for navigating over a variable-arity tree, that is a tree that may have any number of children per node.

That is, I allow one to navigate arbitrarily over a tree structure (a ZTree), and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".!

!TreeZipper class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 16:36'!
on: aTree
	^ self new focus: aTree trail: nil.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 2/9/2011 09:15'!
change: aBlock
	"aBlock is a unary block whose evaluated value we use to replace the current focus."
	^ self changeTo: (aBlock value: focus value).! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 12/31/2010 19:32'!
changeTo: newValue
	^ TreeZipper new
		focus: (ZTree
			value: newValue
			children: focus children)
		trail: trail.! !

!TreeZipper methodsFor: 'copying' stamp: 'fbs 2/16/2011 07:45'!
copy
	^ TreeZipper new
		focus: self focus
		trail: self trail.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 17:17'!
delete
	trail ifNil: [^ ZipperNavigation signal: 'Delete at top'].
	
	(trail leftNodes isEmpty and: [trail rightNodes isEmpty]) ifTrue:
		[^ TreeZipper new
			focus: (ZTree value: trail value) 
			trail: trail path].
			
	trail leftNodes isEmpty ifTrue:
		[^ TreeZipper new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: trail leftNodes
				rightNodes: trail rightNodes allButFirst)].
			
	^ TreeZipper new
		focus: trail leftNodes first
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes allButFirst
			rightNodes: trail rightNodes).! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:33'!
down
	"Return a Zipper focusing on the first child element of this node."
	^ self move: #safeDown.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:37'!
downOrElse: aBlock
	^ self move: #safeDown orElse: aBlock.! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 22:06'!
focus
	^ focus.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 12/31/2010 16:37'!
focus: aTree trail: aTreeZipperNode
	focus := aTree.
	trail := aTreeZipperNode.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 2/25/2011 17:52'!
insertDown: anObject
	"Insert anObject as a child node of the current focus. Sometimes you need a 'spacer' in your tre: a binary tree needs one so that the node can have just a right subtree. In that event, we insert a ZTree empty."

	^ focus isEmpty
		ifTrue: [self changeTo: anObject ]
		ifFalse: [ TreeZipper new
			focus: (anObject = ZTree empty
				ifTrue: [ ZTree empty ]
				ifFalse: [ ZTree value: anObject ])
			trail: (TreeZipperNode
				path: trail
				value: focus value
				leftNodes: OrderedCollection new
				rightNodes: focus children) ]! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 15:49'!
insertLeft: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: (trail leftNodes copy addFirst: (ZTree value: anObject); yourself)
			rightNodes: trail rightNodes)! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 00:10'!
insertRight: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes
			rightNodes: (trail rightNodes copy addFirst: (ZTree value: anObject); yourself))! !

!TreeZipper methodsFor: 'testing' stamp: 'fbs 2/25/2011 17:35'!
isEmpty
	^ focus isEmpty.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
left
	^ self move: #safeLeft.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
leftOrElse: aBlock
	^ self move: #safeLeft orElse: aBlock.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector
	"Move in some direction. If we fall off the data structure, raise an exception."
	| move |
	move := self perform: directionSelector.
	move first = #success ifFalse: [ ^ ZipperNavigation signal: move first ].
	
	^ move second.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector orElse: aBlock
	"Move in some direction. If we can't, run aBlock. If aBlock takes parameters, pass in the current focus as an argument."
	| move |
	move := self perform: directionSelector.
	^ move first = #success
		ifTrue: [ move second ]
		ifFalse: [ aBlock cull: self focus ].! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
right
	^ self move: #safeRight.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
rightOrElse: aBlock
	^ self move: #safeRight orElse: aBlock.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/31/2010 23:29'!
root
	"Return the possibly new/mutated tree under navigation"
	^ trail
		ifNil: [focus]
		ifNotNil: [self up root]! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/25/2011 08:11'!
safeDown
	"Return a pair whose first element tells us whether we successfully went down (represented by the value #success), and a Zipper. If yes, a Zipper focusing on the first child element of this node. If not, a Zipper focused on where we are."
	| success zipper |
	success := self focus children isEmpty
		ifTrue: [ #downAtLeafNode ]
		ifFalse: [ #success ].
	
	zipper := (success = #success) ifTrue:
		[self class new
			focus: focus children first
			trail: (TreeZipperNode
					path: trail
					value: focus value
					leftNodes: OrderedCollection new
					rightNodes: focus children allButFirst)
		] ifFalse: [self copy].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/25/2011 08:11'!
safeLeft
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	trail
		ifNil: [ success := #leftAtRootNode ]
		ifNotNil: [trail leftNodes ifEmpty: [ success := #leftAtLeftmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success) ifTrue:
		[self class new
			focus: trail leftNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes allButFirst)
				rightNodes: (trail rightNodes addFirst: focus; yourself))]
		ifFalse: [ self copy ].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/25/2011 08:11'!
safeRight
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	trail
		ifNil: [ success := #rightAtRootNode ]
		ifNotNil: [ trail rightNodes ifEmpty: [ success := #rightAtRightmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success)
		ifTrue: [ self class new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes addFirst: focus; yourself)
				rightNodes: trail rightNodes allButFirst) ]
		ifFalse: [ self copy ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/25/2011 08:11'!
safeUp
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	success := trail
		ifNil: [ #upAtRootNode ]
		ifNotNil: [ #success ].
	
	zipper := (success = #success)
		ifTrue: [ self class new
			focus: (ZTree
				value: trail value
				children: trail leftNodes reverse, focus asOrderedCollection, trail rightNodes)
			trail: trail path ]
		ifFalse: [ self copy ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 2/16/2011 07:45'!
trail
	^ trail.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
up
	^ self move: #safeUp.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:35'!
upOrElse: aBlock
	^ self move: #safeUp orElse: aBlock.! !

Object subclass: #TreeZipperNode
	instanceVariableNames: 'leftNodes rightNodes path value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipperNode commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a recursive structure that describes a path through a tree.

  self path

 refers to the previous position in the path, itself a TreeZipperNode. The first entry in a path has

  self path == nil

Being part of a zipper, I naturally represent a context-with-a-hole - that part of a tree minus the part currently under focus.!

!TreeZipperNode class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	^ self new
		path: aTreeZipperTrail
		value: anObject
		leftNodes: leftCollection
		rightNodes: rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
leftNodes
	^ leftNodes! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:22'!
path
	^ path.! !

!TreeZipperNode methodsFor: 'private' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	path := aTreeZipperTrail.
	value := anObject.
	leftNodes := leftCollection.
	rightNodes := rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
rightNodes
	^ rightNodes.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 17:06'!
value
	^ value.! !

Object subclass: #ZTree
	instanceVariableNames: 'children value'
	classVariableNames: 'Empty'
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZTree commentStamp: 'fbs 12/30/2010 23:32' prior: 0!
I'm a basic implementation of a tree, only present so that my package doesn't need to rely on a separate tree package.!

ZTree subclass: #ZEmptyTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZEmptyTree commentStamp: 'fbs 2/22/2011 21:41' prior: 0!
I represent an empty tree.!

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
children
	^ #()! !

!ZEmptyTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ true.! !

!ZEmptyTree methodsFor: 'printing' stamp: 'fbs 2/25/2011 17:58'!
printString
	^ '<>'.! !

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
value
	^ nil.! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/22/2011 21:41'!
empty
	^ Empty ifNil: [Empty := ZEmptyTree new].! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/25/2011 17:53'!
value: anObject
	^ anObject = ZTree empty
		ifTrue: [ ZTree empty ]
		ifFalse: [ self new
			value: anObject
			children: OrderedCollection new ]! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	^ self new value: anObject children: aCollection.! !

!ZTree methodsFor: 'comparing' stamp: 'fbs 12/31/2010 00:08'!
= anObject
	^ anObject class == self class and:
		[self value = anObject value and:
			[self children = anObject children]].! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:33'!
children
	^ children! !

!ZTree methodsFor: 'initialize-release' stamp: 'fbs 12/30/2010 23:34'!
initialize
	super initialize.
	children := OrderedCollection new.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ false.! !

!ZTree methodsFor: 'printing' stamp: 'fbs 12/30/2010 23:54'!
printString
	^ '(', self value printString,
		(children
			ifEmpty: ['']
			ifNotEmpty: [' ', (self children reduce: [:a :b | a printString, ', ', b printString])]),
		')'.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:54'!
value
	^ value.! !

!ZTree methodsFor: 'private' stamp: 'fbs 12/31/2010 16:54'!
value: anObject children: aCollection
	value := anObject.
	children := aCollection asOrderedCollection.! !

!ZTree methodsFor: 'enumerating' stamp: 'fbs 12/31/2010 19:21'!
zipper
	^ TreeZipper on: self.! !
